# -*-makefile-*-
#
# Copyright (C) 2013 by Thomas Waldecker <thomas.waldecker@tqs.de>
#
# See CREDITS for details about who has contributed to this project.
#
# For further information about the PTXdist project and license conditions
# see the README file.
#

#
# We provide this package
#
PACKAGES-$(PTXCONF_KERNEL_ARAGO_KERNEL) += kernel-arago-kernel

#
# Paths and names
#
KERNEL_ARAGO_KERNEL_VERSION	:= am33x-$(PTXCONF_KERNEL_ARAGO_KERNEL_GIT_ID)
KERNEL_ARAGO_KERNEL_MD5		:=
KERNEL_ARAGO_KERNEL		:= linux-$(KERNEL_ARAGO_KERNEL_VERSION)
KERNEL_ARAGO_KERNEL_SUFFIX		:= tar.gz
KERNEL_ARAGO_KERNEL_DIR		:= $(BUILDDIR)/linux-arago-kernel-$(KERNEL_ARAGO_KERNEL_VERSION)
KERNEL_ARAGO_KERNEL_CONFIG		:= $(PTXDIST_PLATFORMCONFIGDIR)/kernelconfig-arago-kernel
KERNEL_ARAGO_KERNEL_LICENSE	:= GPLv2
#KERNEL_ARAGO_KERNEL_URL		:= $(call kernel-url, KERNEL_ARAGO_KERNEL)
KERNEL_ARAGO_KERNEL_URL		:= http://arago-project.org/git/projects/?p=linux-am33x.git&a=snapshot&h=$(PTXCONF_KERNEL_ARAGO_KERNEL_GIT_ID)&sf=tgz
KERNEL_ARAGO_KERNEL_SOURCE		:= $(SRCDIR)/$(KERNEL_ARAGO_KERNEL).$(KERNEL_ARAGO_KERNEL_SUFFIX)

# ----------------------------------------------------------------------------
# Prepare
# ----------------------------------------------------------------------------

KERNEL_ARAGO_KERNEL_PATH	:= PATH=$(CROSS_PATH)
KERNEL_ARAGO_KERNEL_ENV		:= KCONFIG_NOTIMESTAMP=1 HOST_EXTRACFLAGS="$(HOST_CPPFLAGS)"
KERNEL_ARAGO_KERNEL_CONF_OPT	:= $(call kernel-opts, KERNEL_ARAGO_KERNEL)

KERNEL_ARAGO_KERNEL_IMAGES := uImage
KERNEL_ARAGO_KERNEL_IMAGES := $(addprefix $(KERNEL_ARAGO_KERNEL_DIR)/arch/$(GENERIC_KERNEL_ARCH)/boot/,$(KERNEL_ARAGO_KERNEL_IMAGES))

ifdef PTXCONF_KERNEL_ARAGO_KERNEL
$(KERNEL_ARAGO_KERNEL_CONFIG):
	@echo
	@echo "*************************************************************************"
	@echo " Please generate a kernelconfig with 'ptxdist menuconfig kernel-arago-kernel'"
	@echo "*************************************************************************"
	@echo
	@echo
	@exit 1
endif

$(STATEDIR)/kernel-arago-kernel.prepare: $(KERNEL_ARAGO_KERNEL_CONFIG)

# ----------------------------------------------------------------------------
# Compile
# ----------------------------------------------------------------------------

KERNEL_ARAGO_KERNEL_MAKE_OPT := \
	$(call kernel-opts, KERNEL_ARAGO_KERNEL) \
	uImage modules

# ----------------------------------------------------------------------------
# Install
# ----------------------------------------------------------------------------

KERNEL_ARAGO_KERNEL_INSTALL_OPT := \
	$(call kernel-opts, KERNEL_ARAGO_KERNEL) \
	modules_install

$(STATEDIR)/kernel-arago-kernel.install:
	@$(call targetinfo)
	@$(call world/install, KERNEL_ARAGO_KERNEL)
	@$(foreach image, $(KERNEL_ARAGO_KERNEL_IMAGES), \
		install -m 644 $(image) $(IMAGEDIR)/$(notdir $(image))-arago-kernel;)
	@$(call touch)

# ----------------------------------------------------------------------------
# Target-Install
# ----------------------------------------------------------------------------

$(STATEDIR)/kernel-arago-kernel.targetinstall:
	@$(call targetinfo)

	@$(call install_init,  kernel-arago-kernel)
	@$(call install_fixup, kernel-arago-kernel, PRIORITY,optional)
	@$(call install_fixup, kernel-arago-kernel, SECTION,base)
	@$(call install_fixup, kernel-arago-kernel, AUTHOR,"Thomas Waldecker <thomas.waldecker@tqs.de>")
	@$(call install_fixup, kernel-arago-kernel, DESCRIPTION,missing)

	@$(call install_copy, kernel-arago-kernel, 0, 0, 0644, \
		$(IMAGEDIR)/uImage-arago-kernel, /boot/uImage-arago-kernel, n)

	@cd $(KERNEL_ARAGO_KERNEL_PKGDIR) && \
		find lib -type f | while read file; do \
			$(call install_copy, kernel-arago-kernel, \
				0, 0, 0644, -, /$${file}, k) \
	done
	@$(call install_finish, kernel-arago-kernel)

	@$(call touch)

# ----------------------------------------------------------------------------
# oldconfig / menuconfig
# ----------------------------------------------------------------------------

kernel-arago-kernel_oldconfig kernel-arago-kernel_menuconfig: $(STATEDIR)/kernel-arago-kernel.extract
	@$(call world/kconfig, KERNEL_ARAGO_KERNEL, $(subst kernel-arago-kernel_,,$@))

# vim: syntax=make
