/*!
    \file main.c
    \brief MBa35-CA mainboard revision example
    \note
    \remarks
    Project: MBa35-CA
    Copyright 2012 TQ Systems GmbH

    TQ Systems GmbH stellt diesen Quelltext zu Dokumentationszwecken zur
    Verfügung. Der Quelltext wird "as is" mit allen enthaltenen Fehlern
    zur Verfügung gestellt. Support wird ausdrücklich nicht vereinbart.
    Der Quelltext darf nicht ohne die ausdrückliche Einwilligung durch
    TQ Systems GmbH weitergegeben werden. Diese Einwilligung bedarf der
    Schriftform.
    Der Quelltext darf nicht ohne die ausdrückliche Einwilligung  durch
    TQ Systems in anderen Projekten weiterverwendet werden. Diese
    Einwilligung bedarf der Schriftform.
*/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <errno.h>

#include <unistd.h>
#include <string.h>
#include <stdio.h>
#include <stdlib.h>

/*!
  \brief get size of an array
  \param TEST_ARAY - base pointer to the array
  \attention do only use on static allocated arrays
*/
#if !defined(ARRAY_SIZE)
#define ARRAY_SIZE(TEST_ARRAY)  (sizeof(TEST_ARRAY) / sizeof(TEST_ARRAY[0]))
#endif

#define REVISION_DETECT0	38 /* IMX_GPIO_NR(2, 6) */
#define REVISION_DETECT1	4  /* IMX_GPIO_NR(1, 4) */
#define REVISION_DETECT2	0  /* IMX_GPIO_NR(1, 0) */

static const char BaseName[] = "/sys/class/gpio/gpio";
static const char ValueNode[] = "value";


/*!
 * \brief read exported gpio (sysfs)
 * \param Idx - gpio index
 * \return 0 if gpio low, 1 if gpio high, -errno otherwise
 * 
 */
int ReadGpio(unsigned Idx)
{
	char File[256];
	char Buffer[32];
	char *p;
	int h;
	ssize_t r;
	long int v;
	
	sprintf(File, "%s%u/%s", BaseName, Idx, ValueNode);
	h = open(File, O_RDONLY);
	if (0 > h) {
		return -(errno);
	}
	r = read(h, Buffer, ARRAY_SIZE(Buffer));
	if (0 < r) {
		v = strtol(Buffer, &p, 10);
		if (!v)
			/*
			 * \attention: strtol returns zero of no number 
			 * could be read or if numer is null. check returned 
			 * parser pointer
			 */
			v = (p != Buffer) ? 0 : -EINVAL;
		else
			v = 1;
	} else {
		/*
		 * return error if nothing could be read
		 */
		v = -ENODATA;
	}
	close(h);
	return v;
}

int main(int argc, char **argv)
{
	/*
	 * Rev 0100: all pins high         -> 0x111 -> 0x000 -> 100
	 * Rev 0200: REVISION_DETECT0 low  -> 0x110 -> 0x001 -> 200
	 * Rev 0300: REVISION_DETECT1 low  -> 0x101 -> 0x010 -> 300
	 */
	unsigned Rev = 0, Revision;
	int r;
	unsigned i;
	static const unsigned Pads[] = {
		REVISION_DETECT0,
		REVISION_DETECT1,
		REVISION_DETECT2,
	};

	for (i = 0; i < ARRAY_SIZE(Pads); ++i)
	{
		r = ReadGpio(Pads[i]);
		switch (r) {
		case 0:
			Rev |= (1 << i);
			break;
		case 1:
			break;
		default:
			fprintf(stderr, "error getting gpio.\n");
			exit(-1);
		};
	};
	Revision = (Rev + 1) * 100;
	printf("MBa35-CA revision code: 0x%x -> Revision %u\n", Rev, Revision);

	return 0;
}

