/*!
    \file AudioEnum.cpp
    \brief Qt Multimedia Audio device enumeration example
    \note
    \remarks
    Copyright 2013 TQ Systems GmbH

    TQ Systems GmbH stellt diesen Quelltext zu Dokumentationszwecken zur
    Verfügung. Der Quelltext wird "as is" mit allen enthaltenen Fehlern
    zur Verfügung gestellt. Support wird ausdrücklich nicht vereinbart.
*/

#include <QAudioDeviceInfo>
#include <QTextStream>

void printInfo(const QAudioDeviceInfo &info)
{
    QTextStream cout(stdout);

    if (info.isNull()) {
        cout << "invalid audio dev info" << endl;
        return;
    }
    if (0 == info.deviceName().length()) {
        cout << "empty audio dev info" << endl;
        return;
    }
    if (info.deviceName() == NULL) {
        cout << "NULL audio dev info" << endl;
        return;
    }
    if (info.deviceName() == "null") {
        cout << "NULL audio dev info" << endl;
        return;
    }
    cout << "CARD: " << info.deviceName() << endl;

    cout << "preferred format:" << endl;
    QAudioFormat format = info.preferredFormat();
    if (format.isValid()) {
        cout << "codec: " << format.codec() << " channels: " << format.channels() << endl;
        cout << "frequency: " << format.frequency() << " sample size: " << format.sampleSize() << endl;
    } else {
        cout << "format: invalid" << endl;
    }

    QStringList codecs = info.supportedCodecs();
    cout << "codecs:" << endl;
    for (QStringList::iterator s = codecs.begin(); s != codecs.end(); ++s) {
        cout << *s << endl;
    }
    cout << "endianess:" << endl;
    QList<QAudioFormat::Endian> endianess = info.supportedByteOrders();
    for (int i = 0; i < endianess.size(); ++i) {
        switch (endianess[i]) {
        case QAudioFormat::BigEndian:
                cout << "big endian" << endl;
                break;
        case QAudioFormat::LittleEndian:
                cout << "little endian" << endl;
                break;
        default:
                cout << "unknown endianess" << endl;
        };
    }
    cout << "channels:" << endl;
    QList<int> channels	= info.supportedChannels();
    for (QList<int>::iterator s = channels.begin(); s != channels.end(); ++s) {
        cout << *s << " ";
    }
    cout << endl;
#if QT_VERSION >= 0x040700
    QList<int> rates = info.supportedSampleRates();
    cout << "sample rates:" << endl;
    for (QList<int>::iterator s = rates.begin(); s != rates.end(); ++s) {
        cout << *s << " ";
    }
    cout << endl;
#else
    QList<int> freqs = info.supportedFrequencies();
    cout << "frequencies:" << endl;
    for (QList<int>::iterator s = freqs.begin(); s != freqs.end(); ++s) {
        cout << *s << " ";
    }
    cout << endl;
#endif
    QList<int> sizes = info.supportedSampleSizes();
    cout << "sample sizes:" << endl;
    for (QList<int>::iterator s = sizes.begin(); s != sizes.end(); ++s) {
        cout << *s << " ";
    }
    cout << endl;
    QList<QAudioFormat::SampleType> types = info.supportedSampleTypes();
    cout << "sample types:" << endl;
    for (int i = 0; i < types.size(); ++i) {
        switch (types[i]) {
        case QAudioFormat::SignedInt:
            cout << "SignedInt" << endl;
            break;
        case QAudioFormat::UnSignedInt:
            cout << "UnSignedInt" << endl;
            break;
        case QAudioFormat::Float:
            cout << "Float" << endl;
            break;
        default:
                cout << "unknown sample type" << endl;
        };
    }

}

void printInfo(QAudio::Mode mode)
{
#ifdef QT_NO_AUDIO_BACKEND
#error
#endif

    QTextStream cout(stdout);
    QList<QAudioDeviceInfo>  devices;
    QString modeStr;
    
    switch (mode) {
    case QAudio::AudioOutput:
    	modeStr = "OUTPUT";
    	break;
    case QAudio::AudioInput:
    	modeStr = "INPUT";
    	break;
    default:
    	modeStr = "UNKNOWN";
    };
    
    cout << "==========================================================================" << endl;
    cout << modeStr << ":" << endl;
    devices = QAudioDeviceInfo::availableDevices(mode);
    for (QList<QAudioDeviceInfo>::iterator s = devices.begin(); s != devices.end(); ++s)
    {
        printInfo(*s);
        cout << "------------------------------------------------------------------------------" << endl;
    }
    cout << "default:" << endl;
    printInfo(QAudioDeviceInfo::defaultOutputDevice());
    cout << "==========================================================================" << endl;
}

