/*!
    \file qsad.cpp
    \brief Qt Multimedia Audio example
    \note
    \remarks
    Copyright 2013 TQ Systems GmbH

    TQ Systems GmbH stellt diesen Quelltext zu Dokumentationszwecken zur
    Verfügung. Der Quelltext wird "as is" mit allen enthaltenen Fehlern
    zur Verfügung gestellt. Support wird ausdrücklich nicht vereinbart.
*/

#include <QtCore/QCoreApplication>
#include <QBuffer>
#include <QTimer>
#include <QStringList>
#include <QFile>
#include <QDebug>
#include <QTextStream>

#include "wav-file.h"
#include "generator.h"
#include "audiosynth.h"

void printInfo(QAudio::Mode mode);

void doHelp(void)
{
    QTextStream cout(stdout);
    
    cout << "This is qsad, a qt sound audio demo" << endl; 
    cout << "(c) 2013 TQ Systems" << endl; 
    cout << "parts of the code are from qt 4.8 demos and examples" << endl; 
    cout << "wav parsing support from project_hindi" << endl; 
    cout << "and are under GPL / BSD style licence" << endl; 
    cout << "======================================================" << endl; 
    cout << "qsad --help: print this help" << endl; 
    cout << "qsad --enum: enumerate devices" << endl; 
    cout << "qsad --tone: generate sine tone" << endl; 
    cout << "qsad <file>: play wave file" << endl; 
    cout << "======================================================" << endl; 
    cout << "note: only unpacked plain PCM wav supported" << endl; 
    cout << "note: code is provided as is with no special warranty" << endl;    
    cout << "note: to handle all errors or play all files" << endl;    
}

int main(int argc, char *argv[])
{
    int result;
    
    QCoreApplication a(argc, argv);
    
    QStringList s = a.arguments();
    if (s.length() < 2) {
    	qWarning() << "missing parameters";
    	doHelp();   	
    	result = -1;
    } else if (s.at(1) == "--help") {
    	doHelp();
    	result = 0;
    } else if (s.at(1) == "--enum") {
    	printInfo(QAudio::AudioOutput);
    	result = 0;
    } else if (QFile::exists(s.at(1))) {   	
	    qDebug() << "try play file ...";
	    WavFile *w = new WavFile(&a);
	    w->openWavFile(s.at(1));
	    w->displayInformation();
	    QTimer *timer = new QTimer(&a);
	    timer->setInterval(500);
	    timer->setSingleShot(true);
            a.connect(timer, SIGNAL(timeout()), w, SLOT(start()));
            a.connect(w, SIGNAL(stopped()), &a, SLOT(quit()));
	    timer->start();
	    result = a.exec();
    } else if (s.at(1) == "--tone") {
	    qDebug() << "try tone generation ...";
            Synthesizer *s = new Synthesizer(&a);		
	    QTimer *timer = new QTimer(&a);
	    timer->setInterval(500);
	    timer->setSingleShot(true);
            a.connect(timer, SIGNAL(timeout()), s, SLOT(start()));
            a.connect(s, SIGNAL(stopped()), &a, SLOT(quit()));
	    qWarning() << "starting ...";
	    timer->start();
	    result = a.exec();  	
    } else {
    	qWarning() << "invalid parameter(s)";
    	doHelp();   	
    	result = -1;
    }
    
    return result;
}

