/* -*- Mode: C; indent-tabs-mode: t; c-basic-offset: 4; tab-width: 4 -*- */
/*
 * project_hindi
 * Copyright (C) Dilawar Singh 2010 <dilawar@ee.iitb.ac.in>
 * Copyright (C) TQ Systems GmbH (adapt to usage with Qt and playing)
 * 
 * project_hindi is free software: you can redistribute it and/or modify it
 * under the terms of the GNU General Public License as published by the
 * Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * project_hindi is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 * See the GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License along
 * with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

#ifndef _WAV_FILE_H_
#define _WAV_FILE_H_

#include <QString>
#include <QObject>
#include <QByteArray>
#include <QTextStream>
#include <QAudioFormat>
#include <QAudioOutput>

#include <cmath>

#include "wav-def.h"

class QFile;
class QAudioOutput;
/*
 * =====================================================================================
 *        Class:  WavFile
 *  Description:  This file is responsible for loading wav file and reading
 *  its content. Wav file data will be loaded into some dat file to be
 *  processed later.
 * =====================================================================================
 */

class WavFile: public QObject
{
private:
    Q_OBJECT
    Q_DISABLE_COPY(WavFile)
    bool isExist;
    bool isOpen;
    bool isClose;
    QString fileName;
    QFile *pFile;
    QTextStream cout;
    WAV_HDR* pWavHeader;
    CHUNK_HDR* pChunkHeader;

public:

    /* ====================  LIFECYCLE     ======================================= */
    WavFile(QObject *parent = 0);                  /* constructor      */
    virtual ~WavFile();                            /* destructor       */

    /* ====================  ACCESSORS     ======================================= */


    /*
     * ===  FUNCTION  ======================================================================
     *         Name:  getNumSamples
     *  Description:  Returns number of samples in file.
     * =====================================================================================
     */
    long int getNumSamples();


    /*
     * ===  FUNCTION  ======================================================================
     *         Name:  getNumChannels
     *  Description:  Returns no of channels (1 == mono, 2 == stereo )
     * =====================================================================================
     */
    int getNumChannels();


    /*
     * ===  FUNCTION  ======================================================================
     *         Name:  getBitsPerSample
     *  Description:  Report the number of bits in each samples.
     * =====================================================================================
     */
    int getBitsPerSample();


    /*
     * ===  FUNCTION  ======================================================================
     *         Name:  getSampleRateHz
     *  Description:  return sample rate in Hz
     * =====================================================================================
     */
    unsigned long getSampleRateHz();


    /*
     * ===  FUNCTION  ======================================================================
     *         Name:  displayInformation
     *  Description:  get all the information about file.
     * =====================================================================================
     */
    int displayInformation();


    /* ====================  MUTATORS      ======================================= */

    /* ====================  OPERATORS     ======================================= */

    /*
     * ===  FUNCTION  ======================================================================
     *         Name:  readCurrentInput()
     *  Description:  Routine for reading one sample from a previously
     *  loaded wave file. Retuens current sample as a double.
     * =====================================================================================
     */
    /* double readCurrentInput(); */
    int readCurrentInput(int& );


    /*
     * ===  FUNCTION  ======================================================================
     *         Name:  ifMoreDataAvailable
     *  Description:  Check if file is over.
     * =====================================================================================
     */
    bool ifMoreDataAvailable();

    //const WavFile& operator = ( const WavFile &other ); /* assignment operator */

    /* ====================  DATA MEMBERS  ======================================= */
private:
    QByteArray gWavDataIn;
    int numInSamples;
    long int maxInSamples;
    QAudioOutput* output;
    QAudioFormat format;

    void reset();
public:

    /*
     * ===  FUNCTION  ======================================================================
     *         Name:  openWavFile
     *  Description:  This will open the wav file for further processing.
     * =====================================================================================
     */
    int openWavFile(const QString& fileName);

    /*
     * ===  FUNCTION ======================================================================
     *         Name:  loadWavData
     *  Description:  This should load the content of wav file in to a
     *  binary file for further processing.
     * =====================================================================================
     */
    int writeDataToFile(const QString&);
    const QByteArray& getData();
    int play(void);

    /*
     *--------------------------------------------------------------------------------------
     *       Class:  WavFile
     *      Method:  closeWavFile
     * Description:  This should close the file after processing.
     *--------------------------------------------------------------------INCLUDES------------------
     */
    int closeWavFile();
private slots:
    void onNotify();
    void onStateChanged(QAudio::State newState);
    void start();
signals:
	void stopped();

}; /* -----  end of class WavFile  ----- */

#endif // _WAV_FILE_H_
