
#include <stdlib.h>
#include <stdio.h>
#include <stdint.h>
#include <fcntl.h>
#include <linux/fb.h>
#include <sys/mman.h>

struct ScreenData
{
	struct fb_var_screeninfo vinfo;
	struct fb_fix_screeninfo finfo;
	long int ScreenSize;	//Number of ??? in the screen... (bytes?)
	int fbfd;
	unsigned char *fbp;
	unsigned char* iBackBuffer;
};	

int initScreen(struct ScreenData* screen)
{
	int fbfd = 0;
	long int screensize = 0;
 	struct fb_var_screeninfo vinfo;
	struct fb_fix_screeninfo finfo;
	char *fbp = 0;
	int x = 0, y = 0;
	long int location = 0;

    /* Open the file for reading and writing */
    fbfd = open("/dev/fb0", O_RDWR);
    if (!fbfd) {
            printf("Error: cannot open framebuffer device.\n");
            exit(1);
    }
    //printf("The framebuffer device was opened successfully.\n");

    /* Get fixed screen information */
    if (ioctl(fbfd, FBIOGET_FSCREENINFO, &finfo)) {
        printf("Error reading fixed information.\n");
        exit(2);
    }

    /* Get variable screen information */
    if (ioctl(fbfd, FBIOGET_VSCREENINFO, &vinfo)) {
            printf("Error reading variable information.\n");
            exit(3);
    }

    /* Figure out the size of the screen in bytes */
    screensize = vinfo.xres * vinfo.yres * vinfo.bits_per_pixel / 8;
		
    /* Map the device to memory */
    fbp = (char *)mmap(0, screensize, PROT_READ | PROT_WRITE, MAP_SHARED, fbfd, 0);       
    if ((int)fbp == -1) { 
		printf("Error: failed to map framebuffer device to memory.\n"); 
		exit(4);
    }
    printf("The framebuffer device was mapped to memory successfully.\n");
    if ((int)vinfo.bits_per_pixel != 16) { 
	printf("The frambuffer's bit depth is %d.\n", vinfo.bits_per_pixel);
		exit(5);
    }

	//Make sure the xoffset and yoffset are zero...
	if (vinfo.xoffset != 0 || vinfo.yoffset != 0)
	{
		printf("Error: The frambuffer's x and y offsets are not zero..\n");
		exit(-1);
	}

	//Clear the screen (the slow way... For no reason in particular... I just want to keep this code around...)
	//(look at clearBackBuffer() to see the fast way...)
	for (y = 0; y < vinfo.yres; y++)
	{
		for (x = 0; x < vinfo.xres; x++)
		{	
			uint16_t* ptr;
			/* Figure out where in memory to put the pixel */
			//location = (x+vinfo.xoffset) * (vinfo.bits_per_pixel/4) + (y+vinfo.yoffset) * finfo.line_length;
//TODO		
			location = (x + vinfo.xoffset) * (vinfo.bits_per_pixel / 8) + 
				( y + vinfo.yoffset) * finfo.line_length;
			ptr = (uint16_t*)(&fbp[location]);
			*ptr = 0x0000;
		}
	}

	//Store our useful screen stuff in the struct.
	screen->finfo = finfo;
	screen->vinfo = vinfo;
	screen->fbp = fbp;
	screen->fbfd = fbfd;
	screen->ScreenSize = screensize;
	
	//Allocate some memory for the backbuffer.
	screen->iBackBuffer = malloc(screensize);
	
	printf("xres: %d, yres: %d, bpp: %d\n", screen->vinfo.xres, screen->vinfo.yres, screen->vinfo.bits_per_pixel);
	printf("xoffset: %d, yoffset: %d\n", screen->vinfo.xoffset, screen->vinfo.yoffset);

	// clearBackBuffer(screen); //Clear our backbuffer.
	
	return 0;
}

//This closes the framebuffer device.
int shutdownScreen(struct ScreenData* screen)
{
	munmap(screen->fbp, screen->ScreenSize);
	close(screen->fbfd);	
	
	free(screen->iBackBuffer);
	return 0;
}

void greyScreen(struct ScreenData* sd)
{
	size_t x, y, s, v, l;
	uint16_t* p = (uint16_t*)sd->fbp;
	
	for (y = 0; y < sd->vinfo.yres; ++y)
	{
		s = y % (1 << 5);
		printf("line: %u, start: %u", y, s);
		for (x = 0; x < sd->vinfo.xres; ++x)
		{
			v = (s << sd->vinfo.red.offset) | 
				(s << (sd->vinfo.green.offset + 1)) | 
				(s << sd->vinfo.blue.offset);
			l = (x + sd->vinfo.xoffset) * (sd->vinfo.bits_per_pixel / 8) + 
			(y + sd->vinfo.yoffset) * sd->finfo.line_length;
			p = (uint16_t*)(&sd->fbp[l]);
			if (0 == x)
			{
				printf(" val[0] = %x, loc %u\n", v, l);
			}
			*p = v;
			++s;
			s %= (1 << 5);
		}
	}
}

void colorScreen(struct ScreenData* sd, size_t Bits, size_t Offset)
{
	size_t x, y, s, v, l;
	uint16_t* p = (uint16_t*)sd->fbp;

	for (y = 0; y < sd->vinfo.yres; ++y)
	{
		s = y % (1 << Bits);
		printf("line: %u, start: %u", y, s);
		for (x = 0; x < sd->vinfo.xres; ++x)
		{
			v = s << Offset;
			l = (x + sd->vinfo.xoffset) * (sd->vinfo.bits_per_pixel / 8) + 
			(y + sd->vinfo.yoffset) * sd->finfo.line_length;
			p = (uint16_t*)(&sd->fbp[l]);
			if (0 == x)
			{
				printf(" val[0] = %x, loc %u\n", v, l);
			}
			*p = v;
			++s;
			s %= (1 << Bits);
		}
	}
}

void colorBar(struct ScreenData* sd)
{
	size_t x, y, s, v, l, w, m;
	uint16_t* p = (uint16_t*)sd->fbp;

	size_t ye = ((((1 << sd->vinfo.red.length) - 1) << sd->vinfo.red.offset) | 
		(((1 << sd->vinfo.green.length) - 1) << sd->vinfo.green.offset));
		
	m = (1 << sd->vinfo.red.length);
	w = sd->vinfo.xres / m;
	printf("w = %u val = %u\n", w, m);
	for (y = 0; y < sd->vinfo.yres / 4; ++y)
	{
		for (x = 0; x < sd->vinfo.xres; ++x)
		{
			s = x / w;
			v = (s < m) ? s << sd->vinfo.red.offset : ye;
			l = (x + sd->vinfo.xoffset) * (sd->vinfo.bits_per_pixel / 8) + 
			(y + sd->vinfo.yoffset) * sd->finfo.line_length;
			p = (uint16_t*)(&sd->fbp[l]);
			*p = v;
		}
	}

	m = (1 << sd->vinfo.blue.length);
	w = sd->vinfo.xres / m;
	printf("w = %u val = %u\n", w, m);
	for (y = sd->vinfo.yres / 4; y < sd->vinfo.yres / 2; ++y)
	{
		for (x = 0; x < sd->vinfo.xres; ++x)
		{
			s = x / w;
			v = (s < m) ? s << sd->vinfo.blue.offset : ye;
			l = (x + sd->vinfo.xoffset) * (sd->vinfo.bits_per_pixel / 8) + 
			(y + sd->vinfo.yoffset) * sd->finfo.line_length;
			p = (uint16_t*)(&sd->fbp[l]);
			*p = v;
		}
	}

	m = (1 << sd->vinfo.green.length);
	w = sd->vinfo.xres / m;
	printf("w = %u val = %u\n", w, m);
	for (y = sd->vinfo.yres / 2; y < sd->vinfo.yres / 4 * 3; ++y)
	{
		for (x = 0; x < sd->vinfo.xres; ++x)
		{
			s = x / w;
			v = (s < m) ? s << sd->vinfo.green.offset : ye;
			l = (x + sd->vinfo.xoffset) * (sd->vinfo.bits_per_pixel / 8) + 
			(y + sd->vinfo.yoffset) * sd->finfo.line_length;
			p = (uint16_t*)(&sd->fbp[l]);
			*p = v;
		}
	}

	m = (1 << sd->vinfo.blue.length);
	w = sd->vinfo.xres / m;
	for (y = sd->vinfo.yres / 4 * 3; y < sd->vinfo.yres; ++y)
	{
		for (x = 0; x < sd->vinfo.xres; ++x)
		{
			s = x / w;
			v = (s << sd->vinfo.red.offset) | 
				(s << (sd->vinfo.green.offset + 1)) | 
				(s << sd->vinfo.blue.offset);
			l = (x + sd->vinfo.xoffset) * (sd->vinfo.bits_per_pixel / 8) + 
				(y + sd->vinfo.yoffset) * sd->finfo.line_length;
			p = (uint16_t*)(&sd->fbp[l]);
			*p = v;
		}
	}

}

int colorFrame(struct ScreenData* sd)
{
	size_t x, y, l, c;
	uint16_t* p;
	
	c = (1 << sd->vinfo.green.length) - 1;
	c <<= sd->vinfo.green.offset;
	for (x = 0; x < sd->vinfo.xres; ++x)
	{
		l = (x + sd->vinfo.xoffset) * (sd->vinfo.bits_per_pixel / 8) + 
		(0 + sd->vinfo.yoffset) * sd->finfo.line_length;
		p = (uint16_t*)(&sd->fbp[l]);
		*p = c;
	}
	c = ((((1 << sd->vinfo.red.length) - 1) << sd->vinfo.red.offset) | 
		(((1 << sd->vinfo.green.length) - 1) << sd->vinfo.green.offset));
	for (x = 0; x < sd->vinfo.xres; ++x)
	{
		l = (x + sd->vinfo.xoffset) * (sd->vinfo.bits_per_pixel / 8) + 
		(sd->vinfo.yres - 1 + sd->vinfo.yoffset) * sd->finfo.line_length;
		p = (uint16_t*)(&sd->fbp[l]);
		*p = c;
	}
	c = (((1 << sd->vinfo.red.length) - 1) << sd->vinfo.red.offset);
	for (y = 0; y < sd->vinfo.yres; ++y)
	{
		l = (0 + sd->vinfo.xoffset) * (sd->vinfo.bits_per_pixel / 8) + 
			(y + sd->vinfo.yoffset) * sd->finfo.line_length;
		p = (uint16_t*)(&sd->fbp[l]);
		*p = c;
	}
	c = (((1 << sd->vinfo.blue.length) - 1) << sd->vinfo.blue.offset);
	for (y = 0; y < sd->vinfo.yres; ++y)
	{
		l = (sd->vinfo.xres - 1 + sd->vinfo.xoffset) * (sd->vinfo.bits_per_pixel / 8) + 
			(y + sd->vinfo.yoffset) * sd->finfo.line_length;
		p = (uint16_t*)(&sd->fbp[l]);
		*p = c;
	}
}

int main(int argc, char* argv[])
{
	struct ScreenData sd;
	if (0 == initScreen(&sd))
	{
		
		if (1 == argc) {
			greyScreen(&sd);
		} else if ( 0 == strcmp(argv[1], "--red")) {
			colorScreen(&sd, sd.vinfo.red.length, sd.vinfo.red.offset);
		} else if ( 0 == strcmp(argv[1], "--green")) {
			colorScreen(&sd, sd.vinfo.green.length, sd.vinfo.green.offset);
		} else if ( 0 == strcmp(argv[1], "--blue")) {
			colorScreen(&sd, sd.vinfo.blue.length, sd.vinfo.blue.offset);
		} else if ( 0 == strcmp(argv[1], "--bar")) {
			colorBar(&sd);
		} else if ( 0 == strcmp(argv[1], "--frame")) {
			colorFrame(&sd);
		} else if ( 0 == strcmp(argv[1], "--help")) {
			printf("%s - a framebuffer test for RGB 565\n", argv[0]);
			printf("(C) TQ Systems GmbH 2010\n");
			printf("usage:\n");
			printf("%s --help\n", argv[0]);
			printf("%s ........... grey stripes\n", argv[0]);
			printf("%s --red ..... red stripes\n", argv[0]);
			printf("%s --green ... green stripes\n", argv[0]);
			printf("%s --blue .... blue stripes\n", argv[0]);
			printf("%s --bar ..... color gradient bars\n", argv[0]);
			printf("%s --frame ... frame at the boarder of the display\n", argv[0]);
		}
	
		
		shutdownScreen(&sd);
	}
	return 0;
}

